#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_icon=lili.ico
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.6.1
 Author:         Thibaut Lauziere
 License: 		GPL v3

 Script Function:
	Detect syslinux version ISOs

#ce ----------------------------------------------------------------------------

Global $found=""
$select = FileSelectFolder("Choose a folder containing ISO files","")
if @error=1 Then Exit
SearchISO($select&"\")
msgBox(0,"Results",$found)

Func DetectSyslinuxInBin($file)
	$file = FileOpen($file, 16)

	If $file = -1 Then
		ConsoleWrite("Could not open syslinux.bin"&@CRLF)
		Return -1
	EndIf
	$content = FileRead($file)
	FileClose($file)

	if StringInStr($content,"49534F4C494E555820342E",2)>0 Then
		ConsoleWrite("Syslinux 4.X detected"&@CRLF)
		Return 4
	Elseif StringInStr($content,"49534F4C494E555820332E",2)>0 Then
		ConsoleWrite("Syslinux 3.X detected"&@CRLF)
		Return 3
	Elseif StringInStr($content,"49534F4C494E555820322E",2)>0 Then
		ConsoleWrite("Syslinux 2.X detected"&@CRLF)
		Return 2
	Elseif StringInStr($content,"49534F4C494E555820312E",2)>0 Then
		ConsoleWrite("Syslinux 1.X detected"&@CRLF)
		Return 2
	Else
		ConsoleWrite("Syslinux version could not be detected"&@CRLF)
		Return 0
	EndIf
EndFunc

Func DetectSyslinuxInISO($iso_file)
	$isolinux_bin = SearchIsolinuxBinInISO($iso_file)

	if StringInStr($isolinux_bin,"NOT FOUND") Then
		ConsoleWrite("Could not find isolinux.bin in "&$iso_file&"( "&$isolinux_bin&" )"&@CRLF)
		Return "Grub ?"
	Else
		ConsoleWrite("Found isolinux.bin in "&$iso_file&"( "&$isolinux_bin&" )"&@CRLF)
		If ProcessExists("7z.exe") > 0 Then ProcessClose("7z.exe")
		$tempfile = @ScriptDir & '\temp'
		FileDelete($tempfile)
		;ConsoleWrite("Extracting isolinux.bin : "&'"' & @ScriptDir & '\tools\7z.exe" e -y "' & $iso_file & '" "'&$isolinux_bin&'" -o"'&$tempfile & '"'&@CRLF)
		$output=_RunReadStd('"' & @ScriptDir & '\tools\7z.exe" e "' & $iso_file & '" "'&$isolinux_bin&'" -y -o"'&$tempfile & '"')
		;ConsoleWrite("Extracting isolinux.bin : "&$output[1]&@CRLF)
		$detected_syslinux = DetectSyslinuxInBin($tempfile&"\isolinux.bin")
		if $detected_syslinux > 0 Then
			Return "Syslinux "&$detected_syslinux
		Else
			Return "Grub ?"
		EndIf

	EndIf
EndFunc

Func SearchISO($folder)
	ConsoleWrite("Searching for ISO in "&$folder&@CRLF)
	$search = FileFindFirstFile($folder&"*")

	; Check if the search was successful
	If $search = -1 Then return 0
	$i=0
	ProgressOn("Searching for ISO and analyzing","")
	While 1
		$file = FileFindNextFile($search)
		If @error Then ExitLoop
		$i+=1
		if StringInStr(FileGetAttrib($folder&$file),"D") Then SearchISO($folder&$file&"\")
		ProgressSet($i*5,"Analyzing "&$file)
		If GetExtension($file)="iso" Then
			$found&=$file&" -> "&DetectSyslinuxInISO($folder&$file)&@CRLF
		EndIf
	WEnd
	ProgressOff()
	; Close the search handle
	FileClose($search)
EndFunc

Func GetExtension($filepath)
	$short_name = StringSplit($filepath, '.')
	if NOt @error AND IsArray($short_name) Then
		Return ($short_name[$short_name[0]])
	Else
		Return "ERROR"
	EndIf
EndFunc


; Create a list of files in ISO
Func SearchIsolinuxBinInISO($iso_to_list)
	Local $line="",$error=""

	$cmd='"' & @ScriptDir & '\tools\7z.exe" l -y -slt "' & $iso_to_list & '"'
	If ProcessExists("7z.exe") > 0 Then ProcessClose("7z.exe")
	$output=_RunReadStd($cmd)
	Switch $output[0]
		Case 0
			;ConsoleWrite("7zip error code 0 : No error"&@CRLF)
			FileWrite(@ScriptDir & "\tools\filelist.txt",$output[1])
			$filelist = FileOpen(@ScriptDir & "\tools\filelist.txt", 0)
			While 1
				$line = FileReadLine($filelist)
				If @error = -1 Then ExitLoop
				If StringInStr($line,"Path =") AND StringInStr($line,"isolinux.bin") Then
					$isolinux_path=StringReplace($line, "Path = ", "")
					FileClose($filelist)
					FileDelete(@ScriptDir & "\tools\filelist.txt")
					Return $isolinux_path
				EndIf
			WEnd
			FileClose($filelist)
			FileDelete(@ScriptDir & "\tools\filelist.txt")
			Return "isolinux.bin Not found"
		Case 2
			ConsoleWrite("7zip error code 2 : Fatal Error"&@CRLF)
			if StringInStr($output[1],"Can not open file as archive") OR StringInStr($output[1],"Cannot open file as archive") Then
				ConsoleWrite($output[1]&@CRLF)
				Return -1
			Else
				ConsoleWrite($output[1]&@CRLF)
				Return -1
			EndIf

		Case 1
			ConsoleWrite("7zip error code 1 : Warning (Non fatal error(s)). For example, one or more files were locked by some other application"&@CRLF)
		Case 7
			ConsoleWrite("7zip error code 7 : Command line error"&@CRLF)
		Case 8
			ConsoleWrite("7zip error code 8 : Not enough memory for operation"&@CRLF)
		Case 255
			ConsoleWrite("7zip error code 255 : User stopped the process"&@CRLF)
	EndSwitch
	Return 0
EndFunc   ;==>InitializeFilesInISO


;===============================================================================
;
; Function Name:   _RunReadStd()
;
; Description::    Run a specified command, and return the Exitcode, StdOut text and
;                  StdErr text from from it. StdOut and StdErr are @tab delimited,
;                  with blank lines removed.
;
; Parameter(s):    $doscmd: the actual command to run, same as used with Run command
;                  $timeoutSeconds: maximum execution time in seconds, optional, default: 0 (wait forever),
;                  $workingdir: directory in which to execute $doscmd, optional, default: @ScriptDir
;                  $flag: show/hide flag, optional, default: @SW_HIDE
;                  $sDelim: stdOut and stdErr output deliminter, optional, default: @TAB
;                  $nRetVal: return single item from function instead of array, optional, default: -1 (return array)
;
;
; Return Value(s): An array with three values, Exit Code, StdOut and StdErr
;
; Author(s):       lod3n
;                  (Thanks to mrRevoked for delimiter choice and non array return selection)
;                  (Thanks to mHZ for _ProcessOpenHandle() and _ProcessGetExitCode())
;                  (MetaThanks to DaveF for posting these DllCalls in Support Forum)
;                  (MetaThanks to JPM for including CloseHandle as needed)
;
;===============================================================================

func _RunReadStd($doscmd,$timeoutSeconds=0,$workingdir=@ScriptDir,$flag=@SW_HIDE,$nRetVal = -1, $sDelim = @CRLF)
    local $aReturn,$i_Pid,$h_Process,$i_ExitCode,$sStdOut,$sStdErr,$runTimer
    dim $aReturn[3]

    ; run process with StdErr and StdOut flags
    $runTimer = TimerInit()
    $i_Pid = Run($doscmd, $workingdir, $flag, 6) ; 6 = $STDERR_CHILD+$STDOUT_CHILD

    ; Get process handle
    sleep(100) ; or DllCall may fail - experimental
    $h_Process = DllCall('kernel32.dll','ptr', 'OpenProcess','int', 0x400,'int', 0,'int', $i_Pid)

    ; create tab delimited string containing StdOut text from process
    $aReturn[1] = ""
    $sStdOut = ""
    While 1
        $sStdOut &= StdoutRead($i_Pid)
        If @error Then ExitLoop
    Wend
    $sStdOut = StringReplace($sStdOut,@cr,@tab)
    $sStdOut = StringReplace($sStdOut,@lf,@tab)
    $aStdOut = StringSplit($sStdOut,@tab,1)
    for $i = 1 to $aStdOut[0]
        $aStdOut[$i] = StringStripWS($aStdOut[$i],3)
        if StringLen($aStdOut[$i]) > 0 then
            $aReturn[1] &= $aStdOut[$i] & $sDelim
        EndIf
    Next
    $aReturn[1] = StringTrimRight($aReturn[1],1)

    ; create tab delimited string containing StdErr text from process
    $aReturn[2] = ""
    $sStderr = ""
    While 1
        $sStderr &= StderrRead($i_Pid)
        If @error Then ExitLoop
    Wend
    $sStderr = StringReplace($sStderr,@cr,@tab)
    $sStderr = StringReplace($sStderr,@lf,@tab)
    $aStderr = StringSplit($sStderr,@tab,1)
    for $i = 1 to $aStderr[0]
        $aStderr[$i] = StringStripWS($aStderr[$i],3)
        if StringLen($aStderr[$i]) > 0 then
            $aReturn[2] &= $aStderr[$i] & $sDelim
        EndIf
    Next
    $aReturn[2] = StringTrimRight($aReturn[2],1)

    ; kill the process if it exceeds $timeoutSeconds
    if $timeoutSeconds > 0 Then
        if TimerDiff($runTimer)/1000 > $timeoutSeconds Then
            ProcessClose($i_Pid)
        EndIf
    EndIf

    ; fetch exit code and close process handle
    If IsArray($h_Process) Then
        Sleep(100) ; or DllCall may fail - experimental
        $i_ExitCode = DllCall('kernel32.dll','ptr', 'GetExitCodeProcess','ptr', $h_Process[0],'int*', 0)
        if IsArray($i_ExitCode) Then
            $aReturn[0] = $i_ExitCode[2]
        Else
            $aReturn[0] = -1
        EndIf
        Sleep(100) ; or DllCall may fail - experimental
        DllCall('kernel32.dll','ptr', 'CloseHandle','ptr', $h_Process[0])
    Else
        $aReturn[0] = -2
    EndIf

    ; return single item if correctly specified with with $nRetVal
    If $nRetVal <> -1 And $nRetVal >= 0 And $nRetVal <= 2 Then Return $aReturn[$nRetVal]

    ; return array with exit code, stdout, and stderr
    return $aReturn
EndFunc